#!/usr/bin/env python3
#
# The emulation test for OpalKelly ADS8568 controller
#

import sys,os
import time
import getopt
import time
import numpy as np

import config.path
import config.ads_conf as ads_conf
from adsmc import *

verbose = False
cnt = 0
sync_err = 0
err_cnt = 0
T = [None, 0]

def cb(i, status, data):
	global cnt, sync_err, err_cnt
	cnt += 1
#	print("{:#x}".format(status[0]), status[1])
	if status[0] & EP.sta_sync_err:
		sync_err += 1
		err_cnt += 1
	t = time.perf_counter()
	if T[0] is None:
		T[0] = t
	else:
		d = t - T[0]
		T[1] = max(T[1], d)
		T[0] = t
	if verbose:
		os.system('cls')
		print( '#%d:\n%s' % (i, ''.join([("%8d" if (i+1) % 8 !=0 else "%8d\n") % x for i, x in enumerate(data)])), flush=True)		
	elif i % 100 == 0:
		print( '!' if sync_err else '.', end='',flush=True)
		sync_err = 0

def main():
	global verbose
	emu = None
	emu_psync_per = None
	try:
		optlist, args = getopt.getopt(sys.argv[1:], 'e:p:vh')
	except getopt.GetoptError:
		err('invalid command line options' + ': ' + repr(sys.argv[1:]))
		return -1

	for opt, arg in optlist:
		if opt == '-v':
			verbose = True
		elif opt == '-e':
			emu = int(arg)
##		elif opt == '-p':
##			emu_psync_per = int(arg)
		elif opt == '-h':
			print( \
"""
ADS8568 OpalKelly controller data acquisition tool. Options available:
  -e N  emulate chain of length N
  -p N  psync per N
  -v    verbose output
  -h    this help
""")
			return 0

	dev = ADSMC(ads_conf)
	if not dev.init(force_load = True):
		print( 'Opal Kelly controller is not connected')
		return -1

	print( dev.info())
	dev.configure(emu)
	started = time.perf_counter()
	try:
		dev.acquire_data(dev.cfg.channels if emu is None else dev.fw_channels(), cb)
	except ControllerError as e:
		print( e )
		return -1
	except KeyboardInterrupt:
		pass

	elapsed = time.perf_counter() - started
	print( '\n%d conversions in %f sec (%d per sec), sync err rate %f%%, max delay %f' \
		% (cnt, elapsed, cnt / elapsed, 100. * err_cnt / cnt, T[1]) )

	return 0

if __name__ == '__main__':
	main()
