#
# The emulation test for OpalKelly ADS8568 controller
#

import config.path

import config.ads_conf as ads_conf
from adsmc import *
import random
import struct
import time
import numpy

class AbortException(Exception):
	pass

dev        = None
max_frames = 10000
frames     = 0
verbose    = False
delay      = 0
chan_data  = None
sync_err   = 0
psync_emu  = 8

start_ts = None
last_pcnt = 0

def cb(i, status, data):
	global sync_err, last_pcnt
	pcnt, pi = status[1] & EP.pcode_cnt_mask, status[1] >> EP.pcode_cnt_bits
	assert pcnt >= last_pcnt
	last_pcnt = pcnt
	if i + 1 >= frames:
		elapsed = time.clock() - start_ts
		print( '\n  %.1f fps, pcnt=%u (%.1f ps), pi=%#x' % (frames / elapsed, pcnt, pcnt / elapsed, pi))
		raise AbortException()
	if status[0] & EP.sta_sync_err:
		sync_err += 1
	if verbose:
		for j, v in enumerate(data):
			chip_n, chan_n, conv_n = v >> 11, (v >> 8) & 7, v & 0xff
			print( '%d[%d] %d %d %d' % (i, j, chip_n, chan_n, conv_n))
	elif i % 100 == 0:
		print( '!' if sync_err else '.',end='')
		sync_err = 0
		dev.set_aux(i / 100)

	assert numpy.all(numpy.bitwise_and(data, 0xff) == i & 0xff)
	assert numpy.all(numpy.bitwise_and(data, 0xff00) == chan_data)
	if delay:
		time.sleep(delay)

def main():
	global dev, frames, chan_data, start_ts, last_pcnt
	dev = ADSMC(ads_conf)
	if not dev.init(force_load = True):
		print( 'Opal Kelly controller is not connected')
		return -1

	print( dev.info())
	channels = dev.fw_channels()

	while True:
		chain  = 1 + random.randrange(0, dev.fw_emulated_chain())
		frames = 1 + random.randrange(0, max_frames)
		print( '\nEmulating chain %u, %u frames' % (chain, frames))
		# prepare reference data
		chan_data = numpy.empty([channels*chain*8], dtype=numpy.uint16)
		for j in range(len(chan_data)):
			chan, lane, stage = j % 8, (j / 8) % channels, chain - j / (8 * channels) - 1
			chan_data[j] = ((stage * channels + lane) << 11) | (chan << 8)

		dev.configure(emu_chain=chain, emu_psync_per=psync_emu)
		try:
			last_pcnt = 0
			start_ts = time.clock()
			dev.acquire_data(channels, cb, dt=numpy.uint16)
		except (ControllerError, KeyboardInterrupt) as e:
			print( e)
			print( 'debug info %#x' % dev.get_debug_info())
			return -1
		except AbortException:
			pass

	return 0

if __name__ == '__main__':
	main()
