#!/usr/bin/env python2
__svn_revision__ = '$Revision: 291 $'

# Get data frame from TS32A

import sys
import getopt
import numpy as np
from time import sleep

import teraFAST.config.pixel_config as pc



from teraFAST.lib.log import *
from teraFAST.lib.common import *


opt_n = 1
opt_p = False

def inf(s):
        if opt_n <= 1:
                msg(l_inf, s)

def notice(s):
        if opt_n <= 1:
                msg(l_notice, s)

#log.inf = inf
#log.notice = notice

class CallbackException(Exception):
    def __init__(self):
        pass

usage =\
"""
TS32A single shot acquisition tool.
Usage:
  ts_get.py [options]
Options available:
  -V          show version  
  -e exp           choose exposure (default 3)
  -r            output raw data (no background compensation, no normalization)
  -l            number of rows to read in single frame
  -n count		number of repetitions for averaging		
  -h          this help
The output consists of data for all (x,y) combinations. There is as many lines as y coordinates and as many
comma-separated data points per line as x coordinates.
See tbus_conf.py, ts_conf.py and ts32a_conf.py, pixel_conf.py for more configuration options.
"""

def main():

        global idx
        global data
        global opt_p
        
        opt_v = False

        def cb(res):
                if res is None:
                        return
                global idx,data
                idx+=1
                np.add(data,res,data)
                if idx >= count:
                        np.divide(data,count,data)
                        printout(data)
                        return True
                else:
                        return False
        

        def printout(data):
                for y in range(len(data[0])):
                        print( ",".join([str(data[x][y]) for x in range(len(data))]))
                pass

        

        try:
                optlist, args = getopt.getopt(sys.argv[1:], 'Vx:y:e:n:l:rhsp')
        except getopt.GetoptError:
                err('invalid command line options' + ': ' + repr(sys.argv[1:]))
                return -1


        xlist=None
        ylist=None
        raw = False
        exp = 3
        count = 1
        lines = 64
        threaded = True
        for opt in optlist:
                if opt[0] == '-V':
                        inf(tools_title + ' ' + ver_title + '.' )
                        opt_v = True
                elif opt[0] == '-p':
                        opt_p = True
                elif opt[0] == '-e':
                        exp = int(opt[1])
                elif opt[0] == '-r':
                        raw = True
                elif opt[0] == '-n':
                        count = int(opt[1])
                elif opt[0] == '-l':
                        lines = int(opt[1])
                elif opt[0] == '-s':
                        threaded = False
                elif opt[0] == '-h':
                        print(usage)
                        return 1
        
       
        
##        threaded = threaded and (count!=1)
        threaded = True
        try:
                from teraFAST import processor as iface
                source = iface.processor(rows=lines,threaded=threaded)

        except RuntimeError:
                print( "RuntimeError: Controller is not connected or busy (another copy running?)" , file=sys.stderr)
                return 1
##        source.SetExposure(exp)

        data = np.zeros((source.X_SIZE,source.Y_SIZE),dtype=float)
        idx = 0                
        source.start(cb)
        while source.proc_thread.is_alive():
                sleep(1)

                
        return 0

if __name__ == '__main__':

        if opt_p:
                import yappi, time
                yappi.set_clock_type("wall")
                yappi.start()
                start =time.clock()
        main()
        if opt_p:
                yappi.stop()
                yappi.get_func_stats().save('stats','pstat')
                print("elapsed ",time.clock()-start," seconds")
                yappi.get_func_stats().strip_dirs().sort('tsub').print_all()
                yappi.get_thread_stats().print_all()
