import numpy as np

__testmode = False


def make_map(channels, fw_channels, chain,dev_type=None,model="A"):
    """Creates a map for demuxing data"""
    if model=="A":
        return phys_A(channels, fw_channels, chain)
    elif model in ["B" , "C", "D", "F","G"]:
        return phys_B(channels, fw_channels, chain)
    elif model=="E":
        return phys_B(channels, fw_channels, chain//2)
    elif model[0:9]=="TestBoard":
        if dev_type is None:
            return phys_Test(channels, fw_channels, chain)
        else:
            return phys_Test(channels, fw_channels, chain, dev = dev_type)
    elif  dev_type=="test":
        return logical(channels, fw_channels, chain)
    elif model in ["3A","3B","3C","3D"]:
        return phys_3(channels, fw_channels, chain)
    else:
##        return None
        return phys_A(channels, fw_channels, chain)


def phys_A(channels, fw_channels, chain):
	"""Creates a map for demuxing data into a physical order for Terasense Superfast rev. 2014.06
		channels - configured number of channels
		fw_channels - firmware number of channels (max channels)
		chain - chain length in each channel"""

	sample_group = fw_channels * 4 
##	aaa = [3,35,2,34,33,1,32,0]
	aaa = [3,sample_group+3,2,sample_group+2,sample_group+1,1,sample_group,0]
# Logical order (1st fw_channel/1st_ADS_in_chain/1st channel in ADS; 1st fw_channel/1st_ADS_in_chain/2nd channel in ADS ...)
	idx = np.array([aaa[j]+ 2*k*sample_group + 4 * i
			for i in range(channels)                                # configured channels (number of chains)
					for k in range(chain)     # chain length
							for j in range(8)       # adc channels   
		], dtype = np.uint16)

# Physical order (chain length per PCB is 8)
	idx = np.take(idx,
		np.array([ (63 - k//2 if k % 2 ==0 else (k-1)//2)  + 64 * m + chain * 8 * i
			for i in range(channels)   # configured channels (number of chains)
				for m in range(chain//8) # chain length per PCB is 8
						for k in range(8*8)
		], dtype = np.uint16))
	return idx

def phys_B(channels, fw_channels, chain):

# Create default index
	assert(channels <= 2)
##        sample_group = fw_channels * 4
##        idx = np.array([(i // 2 if i % 2 == 0 else i // 2 + sample_group) + j * 2 * sample_group
##                        for j in range(chain)
##                                for i in range(channels * 8)], dtype = np.uint16)
##
	sample_group = fw_channels * 4 
	aaa = [3,sample_group+3,2,sample_group+2,sample_group+1,1,sample_group,0]
# Logical order (1st fw_channel/1st_ADS_in_chain/1st channel in ADS; 1st fw_channel/1st_ADS_in_chain/2nd channel in ADS ...)
	idx = np.array([aaa[j]+ 2*k*sample_group + 4 * i
			for i in range(channels)                                # configured channels (number of chains)
					for k in range(chain)     # chain length
							for j in range(8)       # adc channels   
		], dtype = np.uint16)
	aaa = [[0,3,2,1,4,7,6,5],[5,6,7,4,1,2,3,0]]
##	aaa = [[0,1,2,3,4,5,6,7],[7,6,5,4,3,2,1,0]]
	idx = np.take(idx,np.array([i*8+aaa[j][k]+j*8*chain
                                    for i in range(chain)
                                        for k in range(8)    # of adc channels
                                            for j in range(channels-1,-1,-1)
                                            
                            ], dtype = np.uint16))
                                    
        
	return idx


def phys_Test(channels, fw_channels, chain, dev = "adc"):

    # Create default index
	assert(channels <= 2)
##        sample_group = fw_channels * 4
##        idx = np.array([(i // 2 if i % 2 == 0 else i // 2 + sample_group) + j * 2 * sample_group
##                        for j in range(chain)
##                                for i in range(channels * 8)], dtype = np.uint16)
##

	sample_group = fw_channels * 4 
##	aaa = [3,35,2,34,33,1,32,0]
	aaa = [3,sample_group+3,2,sample_group+2,sample_group+1,1,sample_group,0]
# Logical order (1st fw_channel/1st_ADS_in_chain/1st channel in ADS; 1st fw_channel/1st_ADS_in_chain/2nd channel in ADS ...)
	idx = np.array([aaa[j]+ 2*k*sample_group + 4 * i
			for i in range(channels)                                # configured channels (number of chains)
					for k in range(chain)     # chain length
							for j in range(8)       # adc channels   
		], dtype = np.uint16)

    
	if dev == "amp":
		aaa = [[0,3,2,1,4,7,6,5],[5,6,7,4,1,2,3,0]]
	else:
		aaa = [[0,1,2,3,4,5,6,7],[7,6,5,4,3,2,1,0]]
##	
	idx = np.take(idx,np.array([i*8+aaa[j][k]+j*8*chain
                                    for i in range(chain)
                                        for k in range(8)    # of adc channels
                                            for j in range(channels-1,-1,-1)
                                            
                            ], dtype = np.uint16))
                                    
        
	return idx

def logical(channels, fw_channels, chain):
	"""Creates a map for demuxing data into a logical order (1st controller channel/1st_ADS_in_chain/1st channel in ADS; 1st controller channel/1st_ADS_in_chain/2nd channel in ADS ...)
		channels - configured number of channels
		fw_channels - firmware number of channels (max channels)
		chain - chain length in each channel"""	
	sample_group = fw_channels * 4 
##	aaa = [3,35,2,34,33,1,32,0]
	aaa = [3,sample_group+3,2,sample_group+2,sample_group+1,1,sample_group,0]
# Logical order (1st fw_channel/1st_ADS_in_chain/1st channel in ADS; 1st fw_channel/1st_ADS_in_chain/2nd channel in ADS ...)
	idx = np.array([aaa[j]+ 2*k*sample_group + 4 * i
			for i in range(channels)                                # configured channels (number of chains)
					for k in range(chain)     # chain length
							for j in range(8)       # adc channels   
		], dtype = np.uint16)

	return idx

def phys_3(channels, fw_channels, chain):
	"""Creates a map for demuxing data into a logical order (1st controller channel/1st_ADS_in_chain/1st channel in ADS; 1st controller channel/1st_ADS_in_chain/2nd channel in ADS ...)
		channels - configured number of channels
		fw_channels - firmware number of channels (max channels)
		chain - chain length in each channel"""	
	sample_group = fw_channels * 4 

##        idx0 = [(i / 2 if i % 2 == 0 else i / 2 + sample_group) + j * 2 * sample_group
##                    for k in range(channels)
##                        for j in range(chain)
####                                for i in range(channels * 8)
##                            for i in range(8)
##                                
##                ]
##	bbb = [17,19,12,21,29,20,18,14,31,28,8,26,0,1,2,22,30,13,11,27,24,15,10,9,23,5,7,3,4,6,25,16]

	aaa = [3,sample_group+3,2,sample_group+2,sample_group+1,1,sample_group,0]
# Logical order (1st fw_channel/1st_ADS_in_chain/1st channel in ADS; 1st fw_channel/1st_ADS_in_chain/2nd channel in ADS ...)
	idx0 = [aaa[j]+ 2*k*sample_group + 4 * i
                        for i in range(channels)                                # configured channels (number of chains)
                                        for k in range(chain)     # chain length
                                                        for j in range(8)       # adc channels   
                ]

	bbb = [22, 20, 10, 19, 27, 18, 21, 8, 25, 26, 15, 29, 7, 6, 5, 16, 24, 11, 12, 28, 31, 9, 13, 14, 17, 3, 1, 4, 2, 0, 30, 23]


	idx = np.array([idx0[bbb[i]+j*len(bbb)+((k % channels)*len(idx0)//channels)]
                             for j in range((chain*8//len(bbb)))
                                 for k in range(channels) 
                                     for i in range(len(bbb))]
                        ,dtype = np.uint16)
                       
	return idx
