import wx
import random, math

# Default level of black
DEFAULT_BLACK=0

DEFAULT_SCAN_STEP = 0.0015  #m


DEFAULT_RATE = 5000

DEFAULT_SPEED = 10

FRAME_RATE = 30 #fps



# GUI BCG panel class
class BCGPanel(wx.Panel):
    DEFAULT_FRAMELENGTH = 256
    MAX_FRAMELENGTH = 512
    MIN_FRAMELENGTH = 64
    """"Class BCGPanel"""
    def __init__(self,parent,id, main=None,processor = None):

        super(BCGPanel, self).__init__(parent=parent,id=id)

        if not main:
            self.main = parent.GetParent()
        else:
            self.main = main

        
        self.processor = processor

        if self.processor.interface.model in ["E", "3A","3B", "3C","3D"]:
            self.SCAN_STEP = 0.0005
        else:
            self.SCAN_STEP = DEFAULT_SCAN_STEP

        if self.processor.interface.model in ["3A","3B","3C","3D"]:
            self.DEFAULT_RATE = 1000
        else:
            self.DEFAULT_RATE = DEFAULT_RATE
            

        if (self.main.imgSize[0] < self.MAX_FRAMELENGTH):
            self.MAX_FRAMELENGTH = self.main.imgSize[0]
            self.MIN_FRAMELENGTH = self.MAX_FRAMELENGTH//8
        
        vbox = wx.BoxSizer(wx.VERTICAL)
        vbox.Add( (0,10),proportion=0, flag=wx.EXPAND)

        

        vbox.Add(wx.StaticText(self,label="White")) 
        self.white = wx.Slider(self, style=wx.SL_TOP |wx.SL_AUTOTICKS, value=100)
        self.white.SetTickFreq(50)
        self.Bind(wx.EVT_SLIDER, self.OnChangeWhite, id=self.white.GetId())
        vbox.Add( self.white, proportion=0, flag=wx.EXPAND)
        

        vbox.Add(wx.StaticText(self,label="Black")) 
        self.black = wx.Slider(self, style=wx.SL_TOP |wx.SL_AUTOTICKS, value=DEFAULT_BLACK)
        self.black.SetTickFreq(50)
        self.Bind(wx.EVT_SLIDER, self.OnChangeBlack, id=self.black.GetId())
        vbox.Add( self.black, proportion=0, flag=wx.EXPAND)
        

        
        vbox.Add(wx.StaticText(self,label="Gamma"))
        self.gamma = wx.Slider(self, style=wx.SL_INVERSE | wx.SL_TOP |wx.SL_AUTOTICKS, value=50)
        self.gamma.SetTickFreq(50)
        self.Bind(wx.EVT_SLIDER, self.OnChangeGamma, id=self.gamma.GetId())
        vbox.Add( self.gamma, proportion=0, flag=wx.EXPAND)

        hbox = wx.BoxSizer(wx.HORIZONTAL)
        self.autowhite = wx.Button(self,-1, 'Auto W')
        self.Bind(wx.EVT_BUTTON, self.OnAutoWhite, id=self.autowhite.GetId())
        hbox.Add((1,0),proportion=1)
        hbox.Add( self.autowhite, proportion=0, flag=wx.ALIGN_CENTER_VERTICAL)


        self.autoblack = wx.Button(self,-1, 'Auto B')
        self.Bind(wx.EVT_BUTTON, self.OnAutoBlack, id=self.autoblack.GetId())
        hbox.Add((1,0),proportion=1)
        hbox.Add( self.autoblack, proportion=0, flag=wx.ALIGN_CENTER_VERTICAL)
        
        self.resetlevel = wx.Button(self,-1, 'Reset')
        self.Bind(wx.EVT_BUTTON, self.OnResetLevel, id=self.resetlevel.GetId())
        hbox.Add((1,0),proportion=1)
        hbox.Add( self.resetlevel, proportion=0, flag=wx.ALIGN_CENTER_VERTICAL)
        vbox.Add( hbox,proportion=1, flag=wx.EXPAND)
        
        vbox.AddSpacer(20)



        choices = ["Time","Space"]
        self.modeSelect = wx.RadioBox(self,label="Mode", choices=choices, name="modeSelect",majorDimension=2, style=wx.RA_SPECIFY_COLS)
        self.Bind(wx.EVT_RADIOBOX, self.OnModeSelect, id=self.modeSelect.GetId())
        self.modeSelect.SetSelection(0)
        vbox.Add( self.modeSelect, proportion=0, flag=wx.EXPAND)


##        vbox.Add(wx.StaticText(self,label="Rate, lines per second"))
        self.rateLabel = wx.StaticText(self,label="Rate, lines/s")
        self.minRate, self.maxRate = self.processor.GetRateRange()
        self.rate = wx.SpinCtrl(self, min=self.minRate, max=self.maxRate, size=(80, 30), style=wx.TE_PROCESS_ENTER|wx.SP_ARROW_KEYS)
        self.rate.SetValue(self.DEFAULT_RATE)
        self.currentRateValue = int(self.DEFAULT_RATE)
        self.Bind(wx.EVT_SPINCTRL, self.OnChangeRate, id=self.rate.GetId())
        self.Bind(wx.EVT_TEXT_ENTER, self.OnChangeRate, id=self.rate.GetId())
        hbox=wx.BoxSizer(wx.HORIZONTAL)
        hbox.Add(self.rateLabel, proportion=1, flag=wx.ALIGN_CENTER)
##        hbox.Add( wx.StaticText(self,label="%3.0f" % self.minRate), proportion=0, flag=wx.EXPAND)
        hbox.Add( self.rate, proportion=0, flag=wx.ALIGN_CENTER)
##        hbox.Add( wx.StaticText(self,label="%3.0f" % self.maxRate), proportion=0, flag=wx.EXPAND)
        vbox.Add( hbox, proportion=1, flag=wx.EXPAND)

        
        self.maxSpeed = self.rate2speed(self.maxRate)
##        self.minSpeed = 0.1
        self.minSpeed = 0.00001
        DEFAULT_SPEED = self.rate2speed(self.DEFAULT_RATE)
        self.speedLabel = wx.StaticText(self,label="Belt speed, cm/s")
        self.speedSC = wx.SpinCtrlDouble(self, min=(100*self.minSpeed), max=(100*self.maxSpeed),initial=(100*DEFAULT_SPEED), inc = 0.1, size=(80, 30), style=wx.TE_PROCESS_ENTER|wx.SP_ARROW_KEYS)
        self.speedSC.SetValue((100*DEFAULT_SPEED))
        self.currentSpeedSCValue = (100*DEFAULT_SPEED)
        self.Bind(wx.EVT_SPINCTRLDOUBLE, self.OnChangeSpeed, id=self.speedSC.GetId())
        self.Bind(wx.EVT_TEXT_ENTER, self.OnChangeSpeed, id=self.speedSC.GetId())
        hbox=wx.BoxSizer(wx.HORIZONTAL)
        hbox.Add( self.speedLabel, proportion=1, flag=wx.ALIGN_CENTER)
        hbox.Add( self.speedSC, proportion=0, flag=wx.ALIGN_CENTER)
        vbox.Add(hbox, proportion=1, flag=wx.EXPAND)
        self.speedLabel.Enable(False)
        self.speedSC.Enable(False)


        vbox.Add(wx.StaticText(self,label="Subframe length")) 
        self.frameLength = wx.Slider(self, style=wx.SL_TOP |wx.SL_AUTOTICKS, value=self.invFuncFL(self.DEFAULT_FRAMELENGTH), minValue=self.invFuncFL(self.MIN_FRAMELENGTH), maxValue=self.invFuncFL(self.MAX_FRAMELENGTH))
        self.frameLength.SetTickFreq(1)
        self.Bind(wx.EVT_SCROLL, self.OnChangeFrameLength, id=self.frameLength.GetId())
        hbox=wx.BoxSizer(wx.HORIZONTAL)
        hbox.Add( wx.StaticText(self,label="%d" % self.MIN_FRAMELENGTH), proportion=0, flag=wx.EXPAND)
        hbox.Add( self.frameLength, proportion=1, flag=wx.EXPAND)
        hbox.Add( wx.StaticText(self,label="%d" % self.MAX_FRAMELENGTH), proportion=0, flag=wx.EXPAND)
        vbox.Add( hbox, proportion=1, flag=wx.EXPAND)
        


        
        vbox.Add( (0,1),proportion=1, flag=wx.EXPAND)

        self.SetSizerAndFit(vbox)

    def rate2speed(self,x):
        return float(self.SCAN_STEP)*x

    def speed2rate(self,x):
        return float(x)*self.SCAN_STEP
        
    def Initialize(self):    
        self.OnChangeWhite()
        self.OnChangeBlack()
        self.OnChangeGamma()
        self.OnChangeSpeed()
        self.OnChangeFrameLength()
        self.OnChangeRate()
        self.OnModeSelect()
##        self.OnChangeExposure()

    def OnStart(self):
##        self.autolevel.Enable()
##        self.autoexp.Enable()
        self.Initialize()
        
    def OnStop(self):
##        self.autolevel.Disable()
##        self.autoexp.Disable()
        pass

    
    def controlFuncBW(self,x):
        return (float(x)/100)**2
    def invFuncBW(self,x):
        val = int(float(x)**(0.5)*100)
        return (val if val<100 else 100) if val > 0 else 0
    
    def OnChangeWhite(self,event=None):        
        white = float(self.white.GetValue())
        black = float(self.black.GetValue())
        if white < black:
            black = white-1
            self.black.SetValue(black)

        self.main.worker.SetBrightness(self.controlFuncBW(black),self.controlFuncBW(white))
        self.main.worker.SetContrast(self.controlFuncBW(black),self.controlFuncBW(white))
                
    def OnChangeBlack(self,event=None):
        white = float(self.white.GetValue())
        black = float(self.black.GetValue())
        if white < black:
            white = black +1
            self.white.SetValue(white)
        self.main.worker.SetBrightness(self.controlFuncBW(black),self.controlFuncBW(white))
        self.main.worker.SetContrast(self.controlFuncBW(black),self.controlFuncBW(white))

    def controlFuncGamma(self,val):
        val = float((val if val<100 else 100) if val > 0 else 0)
        return math.tan((0.86+((val-50)/80))*math.pi/4)
    def invFuncGamma(self,x):
        val = int(math.atan(x*4/math.pi-0.85)*80+50)
        return (val if val<100 else 100) if val > 0 else 0
    
    def OnChangeGamma(self,event=None):

        val = self.gamma.GetValue()
        self.main.worker.SetGamma(self.controlFuncGamma(val))




    def controlFuncFL(self, x):
        return self.MIN_FRAMELENGTH<<int(x)

    def invFuncFL(self, x):
        i = 0
        x = int(x/self.MIN_FRAMELENGTH)
        while x > 1:
            x>>=1
            i+=1
        return i
    
    def OnChangeFrameLength(self,event=None):
        if isinstance(event, wx.ScrollEvent):
            if(event.GetEventType() == wx.EVT_SCROLL_THUMBTRACK._getEvtType() or event.GetEventType() == wx.EVT_SCROLL_CHANGED._getEvtType()):
                return
        elif event:
            return        
        e = self.controlFuncFL(self.frameLength.GetValue())
        err=self.processor.SetFrameLength(e)
        if err:
            self.main.OnError(err)
        self.main.fps_list = []
        return


    def OnChangeRate(self,event=None):
        if event is not None and self.rate.GetValue() == self.currentRateValue:
            return
        if self.modeSelect.GetStringSelection()=="Time":
            self.currentRateValue = self.rate.GetValue()
            self.rate.Enable(False)
            err=self.processor.SetRate(self.currentRateValue)
            if err:
                self.main.OnError(err)
            self.main.fps_list = []
            wx.CallAfter(self.rate.Enable)
        else:
            pass

    def OnChangeSpeed(self,event=None):
##        if event is not None and self.speedSC.GetValue() == self.currentSpeedSCValue:
##            return
##        if isinstance(event, wx.CommandEvent) :
        if event is not None:
            if not isinstance(event, wx.SpinDoubleEvent) :
                try:
                    x = float(event.String)
                    if x != self.speedSC.GetValue():
                        self.speedSC.SetValue(x)
                except:
                    self.speedSC.SetValue(self.speedSC.GetValue())

            if self.speedSC.GetValue() == self.currentSpeedSCValue:
                return
            
        if self.modeSelect.GetStringSelection()=="Space":
            self.currentSpeedSCValue = self.speedSC.GetValue()
            s = float(self.currentSpeedSCValue)/100
##            if float(s)/self.DEFAULT_RATE < self.SCAN_STEP:
##                r = self.DEFAULT_RATE
##            else:
##                r = self.maxRate
            r = self.maxRate
            if r != self.main.processor.GetRate():
                err=self.processor.SetRate(r)
                if err:
                    self.main.OnError(err)
                self.main.fps_list = []
            RR = (s/r)/self.SCAN_STEP

            if RR < 1:
                fr_l = (1/RR)
                aaa = int(round((r/FRAME_RATE)*RR))
                if aaa == 0:
                    fr_l = int(round((1/RR)))
                else:
                    fr_l = int(round((1/RR))*aaa)
                err = self.processor.SetFrameLength(fr_l)
                if err:
                    self.main.OnError(err)
                
                self.main.worker.SetResizeRatio(RR)
                
            else:
                self.main.worker.SetResizeRatio(RR)
        else:
            pass
                
    def OnModeSelect(self,event=None):
        if self.modeSelect.GetStringSelection()=="Time":
            self.speedLabel.Enable(False)
            self.speedSC.Enable(False)
            self.rateLabel.Enable(True)
            self.rate.Enable(True)
            self.frameLength.Enable(True)
            self.OnChangeFrameLength()
            self.main.worker.SetResizeRatio(1)
            self.OnChangeRate()
        elif self.modeSelect.GetStringSelection()=="Space":
            self.speedLabel.Enable(True)
            self.speedSC.Enable(True)
            self.rateLabel.Enable(False)
            self.rate.Enable(False)
            self.frameLength.Enable(False)
            self.OnChangeSpeed()
        

    def OnAutoWhite(self,event=None):
        data = self.main.worker.source.copy()
        self.white.SetValue(self.invFuncBW(data.max()))
        self.OnChangeWhite()

    def OnAutoBlack(self,event=None):
        data = self.main.worker.source.copy()
        self.black.SetValue(self.invFuncBW(data.min()))        
        self.OnChangeBlack()

    def OnResetLevel(self,event=None):
        self.white.SetValue(100)
        self.OnChangeWhite()
        self.black.SetValue(DEFAULT_BLACK)
        self.OnChangeBlack()
        self.gamma.SetValue(50)
        self.OnChangeGamma()
