import wx
import random, math

# Default level of black
DEFAULT_BLACK=10

# GUI ROI panel class
class ROIPanel(wx.Panel):
    """"Class BCGPanel"""
    def __init__(self,parent,id, main = None):

        super(ROIPanel, self).__init__(parent=parent,id=id)

        if not main:
            self.main = parent.GetParent()
        else:
            self.main = main
            
        vbox = wx.BoxSizer(wx.VERTICAL)
#        vbox.Add( (0,1),proportion=1, flag=wx.EXPAND)


        
        hbox = wx.BoxSizer(wx.HORIZONTAL)
        self.drawROI = wx.CheckBox(self, label="Draw boxes", name="drawROI")
        self.Bind(wx.EVT_CHECKBOX, self.OnDrawROI, id=self.drawROI.GetId())
        self.drawROI.SetValue(True)
        hbox.Add( self.drawROI, proportion=1, flag=wx.EXPAND|wx.TOP)

        self.drawBLOB = wx.CheckBox(self, label="Draw blob", name="drawBLOB")
        self.Bind(wx.EVT_CHECKBOX, self.OnDrawBLOB, id=self.drawBLOB.GetId())
        self.drawBLOB.SetValue(False)
        hbox.Add( self.drawBLOB, proportion=1, flag=wx.EXPAND|wx.TOP)

        self.threshInv = wx.CheckBox(self, label="Inverse", name="threshInv")
        self.Bind(wx.EVT_CHECKBOX, self.OnThreshInv, id=self.threshInv.GetId())
        self.threshInv.SetValue(False)
        hbox.Add( self.threshInv, proportion=1, flag=wx.EXPAND|wx.TOP)
        vbox.Add( hbox,proportion=1, flag=wx.EXPAND)

        

        methods = ["Otsu","Adaptive","Fixed"]
        self.threshMethod = wx.RadioBox(self,label="Method", choices=methods, name="thresh",majorDimension=3,style = wx.RA_SPECIFY_COLS)
        self.Bind(wx.EVT_RADIOBOX, self.OnMethod, id=self.threshMethod.GetId())
        vbox.Add( self.threshMethod,proportion=1, flag=wx.EXPAND)


        vbox.Add(wx.StaticText(self,label="Treshold correction"))
        self.threshLevel = wx.Slider(self, style=wx.SL_TOP |wx.SL_AUTOTICKS|wx.SL_LABELS, value=0, maxValue=255)
        self.threshLevel.SetTickFreq(32)
        self.Bind(wx.EVT_SLIDER, self.OnThresLevel, id=self.threshLevel.GetId())
        vbox.Add( self.threshLevel, proportion=1, flag=wx.EXPAND)

        vbox.Add(wx.StaticText(self,label="Median blur radius"))
        self.threshBlur = wx.Slider(self, style=wx.SL_TOP |wx.SL_AUTOTICKS|wx.SL_LABELS, value=3,  minValue=1, maxValue=7)
        self.threshBlur.SetLineSize(2)
        self.threshBlur.SetTickFreq(2)
        self.Bind(wx.EVT_SLIDER, self.OnThreshBlur, id=self.threshBlur.GetId())
        vbox.Add( self.threshBlur, proportion=1, flag=wx.EXPAND)

        vbox.Add(wx.StaticText(self,label="Block size"))
        self.threshAdArea = wx.Slider(self, style=wx.SL_TOP |wx.SL_AUTOTICKS|wx.SL_LABELS, value=21,  minValue=3, maxValue=63)
        self.threshAdArea.SetLineSize(2)
        self.threshAdArea.SetTickFreq(2)
        self.Bind(wx.EVT_SLIDER, self.OnThreshAdArea, id=self.threshAdArea.GetId())
        vbox.Add( self.threshAdArea, proportion=1, flag=wx.EXPAND)
                

        
        vbox.Add( (0,1),proportion=1, flag=wx.EXPAND)

        
        self.SetSizerAndFit(vbox)


    def Initialize(self):
        self.OnMethod()
        self.OnThresLevel()
        self.OnThreshInv()
        self.OnDrawROI()
        self.OnDrawBLOB()
        self.OnThreshBlur()
        self.OnThreshAdArea()

    def OnStart(self):
        self.Initialize()

    def OnStop(self):
        pass


    def OnMethod(self, event=None):
        if self.main.worker:
            self.main.worker.SetThreshMethod( self.threshMethod.GetSelection() )

    def OnThresLevel(self, event=None):
        if self.main.worker:
            self.main.worker.SetThreshLevel( self.threshLevel.GetValue() )

    def OnThreshInv(self, event=None):
        if self.main.worker:
            if self.threshInv.IsChecked() :
                self.main.worker.processFlags |= self.main.worker.INVERSE_BINARY
            else :
                self.main.worker.processFlags &= ~self.main.worker.INVERSE_BINARY

    def OnDrawROI(self, event=None):
        if self.main.worker:
            if self.drawROI.IsChecked() :
                self.main.worker.processFlags |= self.main.worker.DRAW_ROI
            else :
                self.main.worker.processFlags &= ~self.main.worker.DRAW_ROI
            
    def OnDrawBLOB(self, event=None):
        if self.main.worker:
            if self.drawBLOB.IsChecked() :
                self.main.worker.processFlags |= self.main.worker.DRAW_BLOB
            else :
                self.main.worker.processFlags &= ~self.main.worker.DRAW_BLOB
            
    def OnThreshBlur(self, event=None):
        n = self.threshBlur.GetValue()
        if n % 2 == 0:
            n-=1
            self.threshBlur.SetValue(n)
        if self.main.worker:
            self.main.worker.SetThreshBlur(n)

    def OnThreshAdArea(self, event=None):
        n = self.threshAdArea.GetValue()
        if n % 2 == 0:
            n-=1
            self.threshAdArea.SetValue(n)
        if self.main.worker:
            self.main.worker.SetThreshAdArea(n)
