#! python3
from builtins import str
from builtins import object
#from past.utils import old_div
import sys,os
# from string import strip
from importlib import import_module
import struct
import multiprocessing  
from queue import Full, Empty
import numpy as np
import itertools
import teraFAST.ads.adsmc as adsmc

import teraFAST.ads.config.path
import teraFAST.ads.config.ads_conf as ads_conf
import teraFAST.config.pixel_config as pixel_conf
from teraFAST.lib.ok_ctl import ControllerError

from time import sleep, time, perf_counter

test = False
##test = True
TEST_CHAIN = 4
TEST_CHANNELS = 8
TEST_MODEL = "C"
TEST_R = 0.5
MAX_VALUE = 32768

MIN_RATE,MAX_RATE = 30, 5000  # reads per second

DEFAULT_FRAME_LENGTH = 128

FILTER_THRESHOLD = 128 # frequency of channel errors to be suppressed (i.e., if channel errors occur with at least such spacing they will be suppressed)

QUEUE_LENGTH = 2

MSG_STOP = 0x00
MSG_RESIZE = 0x01
MSG_PERIOD = 0x02
MSG_DIFF = 0x04
MSG_SYNC = 0x08
MSG_EDGE = 0x10
MSG_SYNCDIV = 0x20
MSG_SYNCOUT = 0x40
MSG_HOLDOFF = 0x80
row_idx = 0


class AutoStopException(Exception):
    def __init__(self):
        pass

class NoAcquisitionException(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)

class Config():
    def __init__(self):
        for key, value in ads_conf.__dict__.items():
            if not (key.startswith('__') or key.startswith('_')):
                self.__dict__[key]=value

if test:
    import random
        
def makeTest(length, value, period):
    current_time = perf_counter()
    while (perf_counter() < current_time+period/1000000.):
        pass

    out = np.empty((1,length),dtype=np.int16)
    
    center = (time()*10) % length
    width  = (np.random.random()*0.02+0.1)*length
    out[0,:]=(np.exp(-(np.linspace(0, length, length)-center)**2/width**2)*0.07+0.02)*value
    return  out                                                                           


class deviceEnumerator(object):
    """Creates a list of the available devices. The list should be updated manually if necessary"""
    ctl = None
    devices = {}
    def __init__(self):
        self.update()
        
    def update(self):
        """Updates the device list and returns the list of IDs (serial numbers)."""
        self.ctl = adsmc.ADSMC(ads_conf)
        ids={}
        for i in range(self.ctl.get_dev_count()):
            a = self.ctl.get_ctl_serial(i)
            if not self.ctl.init_core(a):
                continue
            newid = self.ctl.get_id()
            if newid.find('/') != -1:
                newid = newid.split('/')[0]
            elif newid.find('_') != -1:
                newid = newid.split('_')[0]
            else:
                continue
            ids[newid] = {'oksn':a}
            #self.ctl.close()
        self.devices=ids
        del self.ctl
        self.ctl = None
        return sorted(self.devices)

    def devCount(self):
        """Returns the number of devices found during the last update."""
        return len(self.devices)
    
    def getDevList(self):
        """Returns the list of IDs (serial numbers) of the devices found during the last update."""
        return sorted(self.devices)

    def getOKSN(self,sn):
        """Returns serial number of Opal Kelly controller corrsponding to Terasense ID (serial number, like ok101010), if currently available.
Returns empty string otherwise. For internal use."""
        return self.devices.get(sn,{'oksn':''}).get('oksn','')
    


class interface(object):
    def __init__(self,threaded=True):
        self.conf = Config()        
        self.threaded = threaded
        
        self.lock = multiprocessing.Lock() if self.threaded else None
        
        self.queue_length = QUEUE_LENGTH
        
        self.type = ""
        self.model = ""

        
        self.process = None
        self.queue = None
        self.chunk = 1

        self.restart = False

        self.diff = False

        self.period = None

        self.fsync2_bit = 1 << (adsmc.EP.sta_bits_off + adsmc.EP.sta2_fsync2_bit -1)
        
    def init(self, sn = '', t=False):        
        global test
        test = test or t
        if not test:
            
            self.dE = deviceEnumerator()
            if self.dE.devCount() < 1:
                raise RuntimeError('Device is not connected or busy (another copy is running?)' )
                return False
            self.OKSN = self.dE.getOKSN(sn)
            if sn and not self.OKSN:
                raise RuntimeError('Cannot find the device {}. It is not connected or busy.'.format(sn))
                return False

           
            self.ctl = adsmc.ADSMC(self.conf)
            
            if not self.ctl.init_core(port=self.OKSN):
                raise RuntimeError('Device {} is not connected or busy (another copy is running?)'.format(sn) )
                return False
            
            id_string = self.ctl.get_id()
            if id_string.find('/') != -1:
                tokens = id_string.split('/')
            else:
                tokens = id_string.split('_')
                
            self.id = tokens[0].strip()
            self.OKSN = self.dE.getOKSN(self.id)

            if len(tokens) > 1:
                self.type = tokens[1].strip()
            
            if len(tokens) > 2:
                self.model = tokens[2].strip()
                if self.model not in ["A","B","C", "D", "E","F","G", "TestBoard", "TestBoard2","3A","3B","3C","3D","test"]:
                    self.model = ""

            if (self.model == "TestBoard"):
                self.conf.channels = 1
                self.conf.chain_len = 4
                self.conf.fw_path = '../ads/ok/ads_controller_2.bit'
            elif (self.model == "B"):
                self.conf.channels = 2
                self.conf.chain_len = 16
                self.conf.rd_clk_div = 1
                self.conf.fw_path = '../ads/ok/ads_controller_2.bit'
            elif (self.model == "C"):
                self.conf.channels = 2
                self.conf.chain_len = 8
                self.conf.rd_clk_div = 1
                self.conf.fw_path = '../ads/ok/ads_controller_2.bit'
            elif (self.model == "TestBoard2" or self.model == "D" or self.model == "E"):
                self.conf.channels = 2
                self.conf.chain_len = 4
                self.conf.fw_path = '../ads/ok/ads_controller_2.bit'
            elif (self.model == "F"):
                self.conf.channels = 2
                self.conf.chain_len = 32
                self.conf.rd_clk_div = 1                
                self.conf.fw_path = '../ads/ok/ads_controller_2.bit'
            elif (self.model == "G"):
                self.conf.channels = 2
                self.conf.chain_len = 12
                self.conf.rd_clk_div = 1
                self.conf.fw_path = '../ads/ok/ads_controller_2.bit'
            elif (self.model == "3A"):
                if self.type == "1":
                    self.conf.channels = 1
                    self.conf.chain_len = 64
                else:
                    self.conf.channels = 2
                    self.conf.chain_len = 32
                self.conf.rd_clk_div = 2
                self.conf.fw_path = '../ads/ok/ads_controller_2.bit'
                
            elif (self.model == "3B"):
                self.conf.channels = 2
                self.conf.chain_len = 16
                self.conf.rd_clk_div = 2
                self.conf.fw_path = '../ads/ok/ads_controller_2.bit'
            elif (self.model == "3C"):
                self.conf.channels = 2
                self.conf.chain_len = 48 
                self.conf.rd_clk_div = 2
                self.conf.fw_path = '../ads/ok/ads_controller_2.bit'
            elif (self.model == "3D"):
                self.conf.channels = 2
                self.conf.chain_len = 8 
                self.conf.rd_clk_div = 4
                self.conf.fw_path = '../ads/ok/ads_controller_2.bit'
            elif (self.model == "test"):
                if self.type == "1":
                    self.conf.channels = 1
                    self.conf.chain_len = 64
                else:
                    self.conf.channels = 2
                    self.conf.chain_len = 32
                self.conf.fw_path = '../ads/ok/ads_controller_2.bit'

            self.conf.fw_path = '../ads/ok/xem7001/ads_controller_2.bit'
            
            del self.ctl
            self.ctl = adsmc.ADSMC(self.conf)
            self.ctl.init(port=self.OKSN, force_load=True)
            
            
            self.ctl.configure()
            self.controller = "ADS"

#            self.info_string = self.ctl.info()
            self.info_string = "OpalKelly/"+self.controller

            self.X_SIZE = self.ctl.cfg.channels*self.ctl.cfg.chain_len*8
            self.Y_SIZE = 1

            if (self.model == "E"):
                self.X_SIZE = 32

            self.map = pixel_conf.make_map(self.ctl.cfg.channels, self.ctl.fw_channels(), self.ctl.cfg.chain_len, self.type, self.model)
#            del self.ctl

        else:
            self.ctl = None
            self.id = "Test_id"
            self.X_SIZE = TEST_CHANNELS*TEST_CHAIN*8
            self.Y_SIZE = 1
            self.controller = "Test ADS"
            self.info_string = "Test controller"
            self.type = "test"
            self.model = ""
            pixel_conf.make_map(TEST_CHANNELS, TEST_CHANNELS, TEST_CHAIN)
            pass

        

    def info(self,detailed=False):
        return self.info_string
        

    def get_id(self):

        return self.id

    def SetRate(self,rate):
        """Sets interrogation rate, requests per second within 100-5000 range.
"""
        period = int(float(1000000)/(MIN_RATE if rate < MIN_RATE else ( MAX_RATE if rate > MAX_RATE else rate)))
        if self.threaded :
            self.lock.acquire()
        if self.process is not None and self.process.is_alive():            
            self.control.send((MSG_PERIOD,int(period)))
            self.period = period
            self.conf.period = period
            error = None
        else:
            error = self._setperiod(period)
        if self.threaded :
            self.lock.release()
        return error

    def reset(self):
        if test:
            return
        if self.threaded :
            self.lock.acquire()        
        try:
            self.ctl.configure()
        except AttributeError:
            pass
            
        if self.threaded :
            self.lock.release()


    def _setperiod(self,period):
        error = False
        self.period = int(period)
        self.conf.period = int(period)
        if not test:
            try:
                
                self.ctl.configure()
                self.period = self.ctl.cfg.period
            except RuntimeError as e:
                error = e
                self.conf.period = self.period
            except AttributeError as e:
                pass
        return error

    def ExternalSync(self,on):
        """Turns external sync mode on or off.
"""
        if self.threaded :
            self.lock.acquire()
        if self.process is not None and self.process.is_alive():            
            self.control.send((MSG_SYNC,int(bool(on))))
            self.conf.psync_start_en = bool(on)
            error = None
        else:
            error = self._sync(on)
        if self.threaded :
            self.lock.release()
        return error

    def _sync(self,on):
        error = False
        old_period = self.conf.period
        try: 
            old_on  = self.conf.psync_start_en            
        except AttributeError:
            old_on = None
        on = bool(on)
        self.conf.period = 65535 if on else self.period     
##        self.conf.time10mult = False
        self.conf.psync_start_en = on
        
        try:
            self.ctl.configure()
        except RuntimeError as e:
            error = e
            self.conf.period = old_period                
            if old_on is not None:
##                self.conf.time10mult = old_on
                self.conf.psync_start_en = old_on
        except AttributeError as e:
            pass
        
        return error

    def GetExternalSync(self):
        return self.conf.psync_start_en

    def SetEdge(self,rise):
        """Sets mode for external sync, either rising edge (True) or falling edge (False).
"""
        rise = int(bool(rise))
        if self.threaded :
            self.lock.acquire()
        if self.process is not None and self.process.is_alive():            
            self.control.send((MSG_EDGE,rise))
            self.conf.psync_edge = rise
            error = None
        else:
            error = self._edge(rise)
        if self.threaded :
            self.lock.release()
        return error
                              
    def GetEdge(self):
        return self.conf.psync_edge    
                              
    def _edge(self,rise):
        rise = bool(rise)
        self.conf.psync_edge = 1 if rise else 0
        return None

    def SetSyncDivider(self,div=1):
        """Sets divider for external sync signal frequency, 1 <= div <= 32768
"""
        div = int(div)
        div = 1 if div < 1 else (32768 if div > 32768 else div)
        
        if self.threaded :
            self.lock.acquire()
        if self.process is not None and self.process.is_alive():            
            self.control.send((MSG_SYNCDIV,div))
            self.conf.psync_div = div
            error = None
        else:
            error = self._syncdiv(div)
        if self.threaded :
            self.lock.release()
        return error

    def _syncdiv(self,div):
        div = int(div)
        self.conf.psync_div = 1 if div < 1 else (32768 if div > 32768 else div)
        return None
                              
    def GetSyncDivider(self):
        return self.conf.psync_div


    def SetSyncHoldoff(self,n=1):
        """Sets holdoff time for external sync signal. Time in microseconds is given by 2^n.
"""
        n = int(n)
        n = 0 if n < 0 else (15 if n > 15 else n)
        
        if self.threaded :
            self.lock.acquire()
        if self.process is not None and self.process.is_alive():            
            self.control.send((MSG_HOLDOFF,n))
            self.conf.psync_debounce = n
            error = None
        else:
            error = self._holdoff(n)
        if self.threaded :
            self.lock.release()
        return error

    def _holdoff(self,n):
        n = int(n)
        self.conf.psync_debounce = 0 if n < 0 else (15 if n > 15 else n)
        return None
                              
    def GetSyncHoldoff(self):
        return self.conf.psync_debounce


    def SetSyncOut(self,on = True):
        """Turns external sync mode on or off.
"""
        if self.threaded:
            self.lock.acquire()
        if self.process is not None and self.process.is_alive():            
            self.control.send((MSG_SYNCOUT,int(bool(on))))
            self.conf.fsync2_en = bool(on)
            error = None
        else:
            error = self._syncout(on)
        if self.threaded :
            self.lock.release()
        return error

    def _syncout(self,on):
        self.conf.fsync2_en = bool(on)
        return None
                              
    def GetSyncOut(self):
        return bool(self.conf.fsync2_en)



    def SetFrameLength(self, length):
        self.chunk = int(length) if length >= 1 else DEFAULT_FRAME_LENGTH
        
        if self.process is not None and self.process.is_alive():            
            self.control.send((MSG_RESIZE,self.chunk))

    def GetRate(self):
        try:
            return 1000000./self.conf.period
        except AttributeError as e:
            return 0
##        return 0

    def GetRateRange(self):
##        return (0,len(self.timing)-1)
        return (MIN_RATE,MAX_RATE)

    def GetIntTime(self):
        return self.ctl.cfg.period

    def GetDimensions(self):
        """ Returns line size """
        return (self.X_SIZE,self.Y_SIZE)

    def read(self, timeout=1):
        """ Reads data from controller into a new numpy.ndarray and returns it.
The function acquires interface.lock for the duration of its operation if threaded."""

        if self.process is not None and self.process.is_alive():
            self.lock.acquire()
            try:
                data = self.queue.get(timeout=timeout)
            except Empty as error:
                self.lock.release()
                print(str(error),file=sys.stderr)
                return None

#make sure we get the last object in the queue
            try:
                while True:
                    data=self.queue.get_nowait()
            except Empty:
                pass
            self.lock.release()
        else:
            raise NoAcquisitionException("Acquisition process is not running!")
        return data


        
    def read2buffer(self,data, timeout=1):
        """ Reads data from controller into a buffer.
Buffer should be a numpy.ndarray with dtype int16 and appropriate size.
The function acquires interface.lock for the duration of its operation if threaded."""
  
        assert data.shape[0] == self.X_SIZE
        assert data.shape[1] == self.chunk*self.Y_SIZE

        try:
            np.copyto(data,self.read(timeout))                                           # actual readout
        except Exception as e:
            raise
            return e

        return False

##    def auto2buffer(self,data, callback, chunk=1):
##        
##        def proxy(i, status, buf):
##            global row_idx
##            if row_idx ==0:
##                if self.threaded:
##                    self.lock.acquire()
##            np.copyto(data[:,chunk-row_idx-1],buf)
##            row_idx = (row_idx+1)%chunk
##            if row_idx ==0:
##                if self.threaded:
##                    self.lock.release()
##                return callback()
##            else:
##                return
##
##        global row_idx
##        row_idx=0
##        if test:
##            i=0
##            while True:
##                if proxy(i,0,makeTest(self.X_SIZE,MAX_VALUE)):
##                    break
##                i+=1
##        else:
##            self.ctl.acquire_data(self.ctl.cfg.channels,proxy)

## Multiprocessing stuff            



    def run(self, queue=None, control=None, period=None, chunk=DEFAULT_FRAME_LENGTH, diff=False, sync = None, edge = None, sn='', test = False, syncdiv = None, holdoff = None, syncout = None):
        
        def proxy(i, status, buf):
            global row_idx, run_data, run_size, last_line, last_idx, diff, even, error_filter
##            if row_idx ==0:
            if control.poll():
                message, payload = control.recv()
                if message & MSG_STOP:
                    return True
                elif (message & MSG_RESIZE) and payload > 0:
                    run_size = payload
                    run_data =  np.empty((self.X_SIZE,self.Y_SIZE*run_size),dtype=np.int16)
                    row_idx = 0
                elif (message & MSG_PERIOD) and payload > 0:
                    self._setperiod(payload)
                    self.restart=True
                    return True
                elif (message & MSG_DIFF):
                    diff = bool(payload)
                    row_idx = 0
                    even = status[0] & self.fsync2_bit
                    last_line =  np.zeros(self.X_SIZE,dtype=np.int16)
                elif (message & MSG_SYNC):
                    self._sync(payload)
                    last_idx = 0
                    self.restart=True
                    return True
                elif (message & MSG_EDGE):
                    self._edge(payload)
                elif (message & MSG_SYNCDIV):
                    self._syncdiv(payload)
                elif (message & MSG_SYNCOUT):
                    self._syncout(payload)
                elif (message & MSG_HOLDOFF):
                    self._holdoff(payload)
                    
                            
##                        self.restart=True
##                        return True

            if self.conf.psync_start_en and (status[1] == last_idx):
                return
            else:
                last_idx = status[1]
            if diff:
                if even ==  (status[0] & self.fsync2_bit):
                  np.copyto(last_line,buf)
                  return
                elif even:
                  temp = buf-last_line
                else:
                  temp = last_line-buf
                np.copyto(run_data[:,run_size-row_idx-1],temp)
                np.copyto(last_line,buf)
                even =  (status[0] & self.fsync2_bit)
            else:    
                np.copyto(run_data[:,run_size-row_idx-1],buf)
            row_idx = (row_idx+1)%run_size
            if row_idx ==0:
                try:
                    queue.put_nowait(run_data.copy())
                except Full:
                    pass
            if error_filter is not None:
                error_filter +=1
                if error_filter > FILTER_THRESHOLD:
                    error_filter = None
            return

        
        global row_idx, run_data, run_size, last_line, last_idx, even, error_filter

        self._setperiod(period)
        if sync is not None:
            self._sync(sync)
        if edge is not None:
            self._edge(edge)
        if syncdiv is not None:
            self._syncdiv(syncdiv)
        if syncout is not None:
            self._syncout(syncout)
        if holdoff is not None:
            self._holdoff(holdoff)
        self.init(sn, test)
        self.diff_mode(diff)
        
        
        row_idx=0
        last_idx = 0
        even = 0
        run_size = chunk
        run_data =  np.empty((self.X_SIZE,self.Y_SIZE*run_size),dtype=np.int16)
        last_line =  np.zeros(self.X_SIZE,dtype=np.int16)
        diff = self.diff
        error_filter = None
        if test:
            i=0
            self.restart = True
            while self.restart:
                self.restart=False
                while True:
                    if proxy(i,(0 if i%2 == 0 else self.fsync2_bit,0),makeTest(self.X_SIZE,MAX_VALUE, self.conf.period)):
                        break
                    i+=1            
        else:
            self.restart = True
            while self.restart == True:
                self.restart = False
                try:
                    self.ctl.acquire_data(self.ctl.cfg.channels,proxy, idx=self.map)
                except ControllerError as e:
                    if str(e).startswith('controller reported channel error') and error_filter is None:
                        print(str(e),file=sys.stderr)
                        error_filter = 0
                        self.restart = True
                    else:
                         raise

                
    def start(self,chunk = None):
        if self.process is not None and self.process.is_alive():
            return self.queue
        if self.ctl is not None:
            del self.ctl
            self.ctl = None
# to fix embedded environment
        if not ("python" in sys.executable):
            multiprocessing.set_executable(os.path.join(sys.exec_prefix, 'pythonw.exe'))
            sys.argv = [os.path.join(os.path.dirname(__file__),"iface.py")]
        

        self.chunk = chunk if chunk is not None else self.chunk
        queue = multiprocessing.Queue(self.queue_length)
        self.control, ctrl = multiprocessing.Pipe()
        if self.period is None:
            self.period = self.conf.period
        self.process = multiprocessing.Process(target = self.run, args=(queue, ctrl, self.period, self.chunk, self.diff, False, self.conf.psync_edge, self.id, test, self.conf.psync_div, self.conf.psync_debounce, self.conf.fsync2_en))
        
        self.process.start()
        while not self.process.is_alive():
            sleep(0.1)
        self.queue = queue
        self.read()
        if self.conf.psync_start_en:
            self.ExternalSync(True)
            try:
                while True:
                    data = self.queue.get_nowait()
            except Empty as error:
                pass      
        return queue

    def stop(self):
        if self.process is not None and self.process.is_alive():
            self.control.send((MSG_STOP,0))
            self.process.join(0.1)
            if self.process.is_alive():
                self.process.terminate()            
        del self.process
        self.process=None

    def diff_mode(self, on):
        if on:
            self.diff = True
            if self.process is not None and self.process.is_alive(): 
                self.control.send((MSG_DIFF,1))
        else:
            self.diff = False
            if self.process is not None and self.process.is_alive(): 
                self.control.send((MSG_DIFF,0))
        

