__svn_revision__ = '$Revision: 432 $'

import pygtk
pygtk.require('2.0')
import gtk
import gobject

import matplotlib
from matplotlib.figure import Figure
from matplotlib.axes import Subplot
from matplotlib.backends.backend_gtk import FigureCanvasGTK
from matplotlib.backends.backend_gtk import NavigationToolbar2GTK

import warnings
import traceback
from common import *
from log import *

#
# The following class implements generic matplotlib window for data displaying.
# Note that if you need some activity it should be done in worker callback since run()
# will not return until the window closed.
#

class Display:
	def __init__(self, key_hook, worker, msec_interval):

		self.wnd = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.title = suite_title + ' ' + ver_title + '.' + str(revision)
		self.set_title()

		self.wnd.connect("delete_event", self.delete_event)
		self.wnd.connect("destroy", self.quit)

		self.box     = gtk.VBox()
		self.fig     = Figure()
		self.axis    = self.fig.add_subplot(111)
		self.canvas  = FigureCanvasGTK(self.fig)
		self.navbar  = NavigationToolbar2GTK(self.canvas, self.wnd)

		self.box.pack_start(self.canvas, True,  True)
		self.box.pack_start(self.navbar, False, False)
		self.wnd.add(self.box)

		if key_hook:
			self.wnd.connect("key-press-event", self.key_press_event)

		self.key_hook     = key_hook
		self.worker       = worker
		self.worker_      = gobject.timeout_add(msec_interval, self.tout)
		self.status       = None
		self.active       = True
		self.paused_time  = 0
		self.paused_since = None

	def set_title(self, info = None):
		title = self.title
		if info: title += ' ' + info
		self.wnd.set_title(title)

	def set_initial_size(self, w, h):
		self.box.set_size_request(w, h)

	def delete_event(self, widget, event, data=None):
		return False

	def quit(self, w):
		gobject.source_remove(self.worker_)
		gtk.main_quit(w)

	def tout(self):
		if self.active:
			now = gobject.get_current_time()
			if self.paused_since is not None:
				self.paused_time += now - self.paused_since
				self.paused_since = None
			try:
				self.worker(self, now - self.started - self.paused_time)
			except:
				err('\nexception in worker')
				traceback.print_exc()
				self.wnd.destroy()
				return False
		else:
			if self.paused_since is None:
				self.paused_since = gobject.get_current_time()
		return True

	def key_press_event(self, w, e):
		try:
			self.key_hook(self, e)
		except:
			err('\nexception in key hook')
			traceback.print_exc()
			self.wnd.destroy()

	def run(self):
		warnings.simplefilter('ignore', 'PangoWarning')
		self.started = gobject.get_current_time()
		self.wnd.show_all()
		gtk.main()
