# import cPickle as pickle
from builtins import range
from builtins import object
import pickle as pickle
import time


NONE = 0x0
NORM = 0x1
BG   = 0x2

class RefData(object):
    """ Data type for reference information - background and normalization values
        
    """
    def __init__(self,xsize=1,ysize=1, data=None, name=None, timestamp=None, exposure=0, style=NONE, mask=None):

        self.timestamp = timestamp if timestamp else time.asctime()
        self.name = name if name else self.timestamp
        self.style = style
        self.exposure = exposure
        if data is not None:
            self.data = [[data[i][j] for j in range(len(data[0]))] for i in range(len(data))]
        elif self.style==NORM :
            self.data = [[1 for j in range(ysize)] for i in range(xsize)]
        else :
            self.data = [[0 for j in range(ysize)] for i in range(xsize)]
        self.xsize = len(self.data)
        self.ysize = len(self.data[0])
        
        if self.style==NORM:
            self.mask = mask if mask is not None else [[1 for j in range(ysize)] for i in range(xsize)]

    def save (self, file):
        """ Save data """
        pickler = pickle.Pickler(file)
        pickler.dump(self.name)
        pickler.dump(self.timestamp)
        pickler.dump(self.exposure)
        pickler.dump(self.style)
        pickler.dump(self.xsize)
        pickler.dump(self.ysize)
        pickler.dump(self.data)
        if self.style == NORM :
           pickler.dump(self.mask)

    def load (self, file):
        """ Load data """
        pickler = pickle.Unpickler(file,fix_imports=True,encoding='latin-1')
        self.name = pickler.load()
        self.timestamp = pickler.load()
        self.exposure = pickler.load()
        self.style = pickler.load()
        self.xsize = pickler.load()
        self.ysize = pickler.load()
        self.data = pickler.load()
        if self.style == NORM :
            self.mask = pickler.load()
